@@ -3,6 +3,7 @@ from urllib import unquote
 from urlparse import urljoin
 from datetime import datetime
 from sha import sha
+import operator
 
 from webob import Request, Response
 from simplejson import loads, dumps, JSONEncoder
@@ -23,8 +24,12 @@ class DatetimeEncoder(JSONEncoder):
 
 
 class JSONStore(object):
-    def __init__(self, **kwargs):
-        self.em = EntryManager(**kwargs)
+    def __init__(self, *args, **kwargs):
+        """
+        A REST(ish) interface to the JSON store.
+
+        """
+        self.em = EntryManager(*args, **kwargs)
 
     def __call__(self, environ, start_response):
         req = Request(environ)
@@ -34,29 +39,28 @@ class JSONStore(object):
 
     def GET(self, req):
         path_info = req.path_info.lstrip('/') or '{}'  # empty search
-        path_info = unquote(path_info)
-        obj = load_entry(path_info)
+        tokens = [ load_entry(unquote(token)) for token in path_info.split('/') ]
 
-        jsonp = req.GET.get('jsonp') or req.GET.get('callback')
-        size = req.GET.get('size')
-        offset = req.GET.get('offset')
-
-        if isinstance(obj, (int, long, float, basestring)):
+        # first token is either an id or a search dict
+        obj = tokens.pop(0)
+        if isinstance(obj, dict):
+            obj = replace_operators(obj)
+            x_items = self.em.search(obj, count=True)
+            result = self.em.search(obj, req.GET.get('size'), req.GET.get('offset'))
+        else:
             try:
                 result = self.em.search(__id__=obj)[0]
-                items = 1
-            except IndexError:
+                result = reduce(operator.getitem, [result] + tokens)
+                x_items = 1
+            except (IndexError, KeyError):
                 return Response(status='404 Not Found')
-        else:
-            obj = replace_operators(obj)
-            items = self.em.search(obj, count=True)
-            result = self.em.search(obj, size, offset)
-        
+
         body = dumps(result, cls=DatetimeEncoder)
         etag = '"%s"' % sha(body).hexdigest()
         if etag in req.if_none_match:
             return Response(status='304 Not Modified')
 
+        jsonp = req.GET.get('jsonp') or req.GET.get('callback')
         if jsonp:
             body = jsonp + '(' + body + ')'
             content_type = 'text/javascript'
@@ -67,7 +71,7 @@ class JSONStore(object):
                 body=body,
                 content_type=content_type,
                 charset='utf8',
-                headerlist=[('X-ITEMS', str(items)), ('etag', etag)])
+                headerlist=[('X-ITEMS', str(x_items)), ('etag', etag)])
 
     def HEAD(self, req):
         response = self.GET(req)
@@ -169,5 +173,5 @@ def parse_op(obj):
 
 if __name__ == '__main__':
     from paste.httpserver import serve
-    app = JSONStore('sqlite://test.db')
+    app = JSONStore('test.db')
     serve(app, port=8081)
